% show spectrogram applications to demonstrate the benefits of a
% localized spectral analysis
%
% G.Doblinger, TU-Wien, 11-2009
%

clc, close all;

% start with spectrogram of a sum of sinusoidal signals
% to show frequency localization

% create a sum of sinusoids 
%        Nsin
% x[n] = sum  A_k*sin(2pi*f_k/Fs*n + phi_k)
%        k=1

Fs = 16000;
f = [800 1600 3000]';
A = [1 1 1]';
phi = [0 pi/4 pi/3]';

Nx = 2000;
n = 0:Nx-1;
X = A(:,ones(1,Nx)).*sin(2*pi/Fs*(f*n)+phi(:,ones(1,Nx)));
x = sum(X);
x = x(:);

Nwin = 160;           % no window effect
%Nwin = 128;           % shows window effect
Mov = Nwin/2;         % overlapping of frames
Nfft = Nwin;          % FFT length 
%w = hamming(Nwin);    % removes blocking effects
w = rectwin(Nwin);    % shows blocking effects (if Nwin not a multiple
                      % of period length)

[Xs,f,t] = spectrogram(x,w,Mov,Nfft,Fs);  % compute spectrogram

close all;
pos = [0.2 0.4 0.6 0.6];
figure('Units','normal','Position',pos);

subplot(2,1,1), plot(n/Fs,x), grid on;
xlabel('time in sec.'), ylabel('x[n]');
title('sum of sinusoids');
subplot(2,1,2),imagesc(t,f,abs(Xs)), colormap(jet), colorbar;
set(gca,'YDir','normal');
xlabel('time in sec.'), ylabel('frequency in Hz');
title(sprintf('mag. spectrogram (lin. scale), Nwin = %d, Nfft = %d, Mov = %d',...
   Nwin,Nfft,Mov));

%%
%
% spectrogram of 1-impulses to show time localization

Fs = 16000;
Nx = 2000;
n = 0:Nx-1;
ni = [200 400 500];  % position of 1-impulses
x = zeros(1,Nx);
for k = 1:length(ni)
   x = x + double(n == ni(k)); 
end
Nwin = 16;           % small window for high time resolution
Mov = Nwin/2;
Nfft = 256;
%w = hamming(Nwin);   % influence on spectral amplitudes
w = rectwin(Nwin);   % rectangular window does not modify impulses 
[Xs,f,t] = spectrogram(x,w,Mov,Nfft,Fs);

close all;
pos = [0.2 0.4 0.6 0.6];
figure('Units','normal','Position',pos);

subplot(2,1,1), plot(n/Fs,x), grid on;
xlabel('time in sec.'), ylabel('x[n]');
title('sum of 1-impules');
subplot(2,1,2), imagesc(t,f,abs(Xs)), colormap(jet), colorbar;
set(gca,'YDir','normal');
xlabel('time in sec.'), ylabel('frequency in Hz');
title(sprintf('mag. spectrogram (lin. scale), Nwin = %d, Nfft = %d, Mov = %d',...
   Nwin,Nfft,Mov));

%%
%
% spectogram of modulated gaussian impulses (i.e. impulses shifted both
% in time and frequency)

Fs = 16000;
Nx = 2000;
n = 0:Nx-1;
ni = [200 500 1000 1600 1600];     % positions of impulses
fi = [1000 2000 3000 5000 3000];   % frequencies of impulses
si = [30 60 80 100 200];           % width of impulses
x = zeros(1,Nx);
j = 1i;                  % j = sqrt(-1)
for k = 1:length(ni)
   x = x + real(exp(-((n-ni(k))/si(k)).^2+j*2*pi*fi(k)/Fs*n)); 
end

Nwin = 32;
Mov = Nwin/2;
Nfft = 256;
w = hamming(Nwin);       % avoids side lobes
%w = rectwin(Nwin);

[Xs,f,t] = spectrogram(x,w,Mov,Nfft,Fs);  % compute spectrogram

close all;
pos = [0.2 0.4 0.6 0.6];
figure('Units','normal','Position',pos);

subplot(2,1,1), plot(n/Fs,x), grid on;
xlabel('time in sec.'), ylabel('x[n]');
title('sum of modulated gaussian impulses');
subplot(2,1,2), imagesc(t,f,abs(Xs)), colormap(jet), colorbar;
set(gca,'YDir','normal');
xlabel('time in sec.'), ylabel('frequency in Hz');
title(sprintf('mag. spectrogram (lin. scale), Nwin = %d, Nfft = %d, Mov = %d',...
   Nwin,Nfft,Mov));


%%
%
% 1. spectrogram of a chirp (linearly modulated sinusoidal signal)
% 2. spectrogram of sinusoidal PM/FM
% demonstrate aliasing

Fs = 16000;
Nx = 1000;
n = 0:Nx-1;
mode = 1;
if mode == 1            % chirp
   alpha = 0.5;         % modulation parameter (no aliasing with 0<alpha<=0.5) 
   %alpha = 1;           % aliasing
   x = sin(alpha*pi/Nx*n.^2);
   tstr = 'sinusoidal chirp signal (lin. sweep)';
else                    % sinusoidal PM/FM 
   f0 = 3000;
   fm = 50;
   dphi = 20;
   x = sin(2*pi*f0/Fs*n+dphi*cos(2*pi*fm/Fs*n));
   tstr = 'sinusoidal phase/frequency modulation';
end

Nwin = 64;
Mov = ceil(3*Nwin/4);
Nfft = 256;
w = hamming(Nwin);      % avoids side lobes
%w = rectwin(Nwin);

[Xs,f,t] = spectrogram(x,w,Mov,Nfft,Fs);  % compute spectrogram

close all;
pos = [0.2 0.4 0.6 0.6];
figure('Units','normal','Position',pos);

subplot(2,1,1), plot(n/Fs,x), grid on;
xlabel('time in sec.'), ylabel('x[n]');
title(tstr);
subplot(2,1,2), imagesc(t,f,abs(Xs)), colormap(jet), colorbar;
set(gca,'YDir','normal');
xlabel('time in sec.'), ylabel('frequency in Hz');
title(sprintf('mag. spectrogram (lin. scale), Nwin = %d, Nfft = %d, Mov = %d',...
   Nwin,Nfft,Mov));

%%
%
% spectrogram of a bird song (blackbird)

[x,Fs] = wavread('blackbird.wav');
Nx = length(x);
n = 0:Nx-1;

Nwin = 1024;            % optimum value regarding time/frequency resolution
Mov = ceil(3*Nwin/4);
Nfft = 2*Nwin;
w = hamming(Nwin);

[Xs,f,t] = spectrogram(x,w,Mov,Nfft,Fs);  % compute spectrogram

fmax = 4000;            % limit frequency scale to [0,fmax]
nmax = round(fmax/Fs*Nfft);
f = f(1:nmax);
Xs = Xs(1:nmax,:);
close all;
pos = [0.2 0.4 0.6 0.6];
figure('Units','normal','Position',pos);

subplot(2,1,1), plot(n/Fs,x), grid on;
xlabel('time in sec.'), ylabel('x[n]');
title('blackbird song');

% Note: use image() instead of imagesc() to avoid scaling to full colormap

subplot(2,1,2), image(t,f,abs(Xs)), colormap(jet), colorbar;
set(gca,'YDir','normal');
xlabel('time in sec.'), ylabel('frequency in Hz');
title(sprintf('mag. spectrogram (lin. scale), Nwin = %d, Nfft = %d, Mov = %d',...
   Nwin,Nfft,Mov));

%%
%
% spectrogram of a speech signal

[x,Fs] = wavread('female_1ch_16kHz.wav');
Nx = length(x);
n = 0:Nx-1;

Nwin = 1024;            % optimum value regarding time/frequency resolution
Mov = ceil(3*Nwin/4);
Nfft = 2*Nwin;
w = hamming(Nwin);

[Xs,f,t] = spectrogram(x,w,Mov,Nfft,Fs);  % compute spectrogram
Xmag = abs(Xs);
Xmag = Xmag/max(max(Xmag));
Xmag = max(20*log10(Xmag),-60);   % scale spectrogram to [0,-60dB]

close all;
pos = [0.2 0.4 0.6 0.6];
figure('Units','normal','Position',pos);

subplot(2,1,1), plot(n/Fs,x), grid on;
xlabel('time in sec.'), ylabel('x[n]');
title('female speech signal');

% Note use image() instead of imagesc() to avoid scaling to full colormap

subplot(2,1,2), imagesc(t,f,Xmag), colormap(jet), colorbar;
set(gca,'YDir','normal');
xlabel('time in sec.'), ylabel('frequency in Hz');
title(sprintf('mag. spectrogram (log. scale in dB), Nwin = %d, Nfft = %d, Mov = %d',...
   Nwin,Nfft,Mov));


