% define ecg data processing class
%
% Note: load_ecg_params.m must be called before using this class object
%
% G.Doblinger, TU-Wien, 10-2009

classdef ecg_dsp
   properties
      x_ecg       % ecg measurement data
      fc_lp       % low pass cutoff frequency in Hz
      fc_hp       % high pass cutoff frequency in Hz
      Nfir        % FIR filter length
      name        % name of experiment
   end
   properties (Constant)
      Fs = 250    % sampling frequency in Hz (must not be changed)
   end   
   properties (Dependent)
      x_ecg_bp    % band pass filtered ecg
      heart_beat  % measured heart beat rate
   end
   methods
      function ob = ecg_dsp(x_ecg,fc_lp,fc_hp,Nfir,name)
         % create and load data to the ecg object
         % Note: this is the class constructor, it has the same name as
         %       the class, it has only one output argument
         
         ob.x_ecg = x_ecg;
         ob.fc_lp = fc_lp;
         ob.fc_hp = fc_hp;
         ob.Nfir = Nfir;
         ob.name = name;
      end
      
      function y = bpfilter(ob)
         % compute band pass filtered ecg
         % window design of FIR filter 
         
         h = fir1(ob.Nfir,[2*ob.fc_hp/ob.Fs 2*ob.fc_lp/ob.Fs],'bandpass');
         y = filter(h,1,[ob.x_ecg(:) ; zeros(ob.Nfir,1)]);
         y = y(ob.Nfir+1:end);   % remove filter settling time
      end
         
      function plot(ob)
         % plot original and processed ecg signals
         % Note: this replaces the MATLAB plot function but only if ob is
         % passed as an argument (function overloading)       
         
         N = length(ob.x_ecg);
         t = 1/ob.Fs*(0:N-1);
         close all
         pos = [0.01 0.4 0.5 0.5];
         figure('numbertitle','off','name',ob.name,'Units',...
            'normal','Position',pos);
         subplot(2,1,1), plot(t,ob.x_ecg), grid on;
         xlabel('sec.'), ylabel('original ecg');
%         y = bpfilter(ob);
         y = ob.x_ecg_bp;     % alternative to call of function bffilter()
         subplot(2,1,2), plot(t,y), grid on;
         xlabel('sec.'), ylabel('bp filtered ecg');
         hb = ob.heart_beat;
         title(sprintf('average heart beat = %3.2f 1/min',hb));
      end
      
      function fftplot(ob,Nfft)
         % plot spectrum magnitude of original ecg signal
         % Nfft = optional FFT length
         
         if nargin == 1
            Nfft = length(ob.x_ecg);
         end
         X = fft(ob.x_ecg,Nfft);
         Nfh = Nfft/2+1;
         f = linspace(0,ob.Fs/2,Nfh);
         N = length(ob.x_ecg);
         t = 1/ob.Fs*(0:N-1);
         close all;
         pos = [0.01 0.4 0.5 0.5];
         figure('numbertitle','off','name',ob.name,'Units',...
            'normal','Position',pos);
         subplot(2,1,1), plot(t,ob.x_ecg), grid on;
         xlabel('sec.'), ylabel('original ecg');
         subplot(2,1,2), plot(f,abs(X(1:Nfh))), grid on;
         xlabel('Hz'), ylabel('spectral magnitude');
         title(sprintf('FFT length = %d',Nfft));
      end
         
      function x_ecg_bp = get.x_ecg_bp(ob)
         % perform band pass filtering
         
         x_ecg_bp = bpfilter(ob);
      end
      
      function heart_beat = get.heart_beat(ob)
         % compute average heart beat rate
         
         y = bpfilter(ob);
         yt = zeros(size(y));
         yt(y > 0.75*max(y)) = 1;    % create pulse train
         dyt = diff(yt);             % detect transitions by computing 
                                     % difference between adjacent samples 
         ip = find(dyt > 0);
         in = find(dyt < 0);
         Ni = min(length(ip),length(in));
         Np = sum(ip(2:Ni)-in(1:Ni-1))/(Ni-1);  % sum up intervals between
                                                % pos. and neg. transitions
         heart_beat = 60*Np/ob.Fs;
         fprintf(1,'estimated heart beat = %3.2f 1/min\n',heart_beat);
      end
   end
end
