% filter design example using signal processing toolbox of MATLAB
% (the traditional way)
%
% G.Doblinger, TU-Wien, 11-2009

% bandpass filter specifications

%                ________  
% Ap             ________
%               |        |
%
% As1_______|                |_______As2
%   0       fs1 fp1      fp2 fs2     Fs/2

clear all
clc

Fs = 16000;   % sampling frequency in Hz
fs1 = 1800;   % lower stopband frequency in Hz
fp1 = 2000;   % lower passband frequency in Hz
fp2 = 3000;   % upper passband frequency in Hz
fs2 = 3200;   % upper stopband frequency in Hz
As1 = 80;     % lower stopband attenuation in dB
Ap = 1.0;     % passband ripple in dB
As2 = 80;     % upper stopband attenuation in dB

% FIR filter design

% determine FIR filter order of equiripple design

delta_p = (10^(Ap/20)-1)/(10^(Ap/20)+1);  % passband ripple
delta_s1 = 10^(-As1/20);                  % stopband ripples
delta_s2 = 10^(-As2/20);
[N,f,a,w] = firpmord([fs1 fp1 fp2 fs2],[0 1 0],...
   [delta_s1 delta_p delta_s2],Fs);

% FIR design using Remez (Parks-McClellan algorithm)

h = firpm(N,f,a,w);    % FIR filter impulse response

Hfir = dfilt.dffir(h); % FIR filter structure
disp(Hfir);

close all;
fvtool(Hfir);
%fvtool(h,1);
%freqz(h,1);           % old-fashioned style

%%

% repeat with elliptic IIR filter

Fsh = Fs/2;
As = max(As1,As2);
[N,Wp] = ellipord([fp1/Fsh fp2/Fsh],[fs1/Fsh fs2/Fsh],Ap,As);

[b,a] = ellip(N,Ap,As,Wp);

% determine cascade form of 2nd order sections
% direct form I, norm-2 scaling, poles closest to unit circle at the end

[sos,g] = tf2sos(b,a,'up','two');

Hiir = dfilt.df1sos(sos,g);
disp(Hiir); 

close all
fvtool(Hiir);

%%

% quantize FIR filter coefficients
% Note: quantization probably not supported without filter design toolbox

Nw = 12;             % word length in bits
Hfirq = Hfir;
Hfirq.Arithmetic = 'fixed';
set(Hfirq, 'CoeffWordLength',Nw);

close all;
fvtool(Hfirq);

%%

% repeat with IIR filter

Nw = 12;              % word length in bits
Hiirq = Hiir;
Hiirq.Arithmetic = 'fixed';
set(Hiirq, 'CoeffWordLength',Nw);
fvtool(Hiirq);




