function PlotWavTree(D,pos,figtext)
% function PlotWavTree(D,pos,figtext)
%
% plot wavelet packet tree (binary trees only)
%
% D         tree structure
% pos       figure window position [x_ori,y_ori,x_length,y_length]
%           (normalized figure coordinates)
% figtext   figure text string

% Db, 28.12.97 (based on plottree() of wavelet toolbox)

if nargin == 0
   help PlotWavTree
   return
elseif ~ischar(figtext)
   error('figtext must be a string');
end

Nnodes = length(D.entr);   % number of nodes in the tree (without top node)
if Nnodes == 1
   disp('Tree not plotted for only one node');
   return
end
N = log2(Nnodes+2)-1;      % number of tree levels

% determine node positions in normalized figure window coordinates

xoffset = 0;
dy = 1/N;
xpos = zeros(1,Nnodes);
ypos = zeros(1,Nnodes);
y0 = 0;
for n = N:-1:1
   inodes = 2^n-1:2^(n+1)-2;
   xpos(inodes) = linspace(xoffset,1-xoffset,2^n);
   ypos(inodes) = y0;
   y0 = y0 + dy;
   xoffset =  (xpos(inodes(1))+xpos(inodes(2)))/2;
end

% coordinates of top node

xtpos = 0.5;
ytpos = 1;

% open figure window and create figure axis

ftree = figure('Units','normalized','Position',pos,'Name',[figtext, ' (xy-zoom on)']);

line_color_tp = 'b';
line_color_hp = 'r';
txt_color  = 'k';
txt_font = 9;
pos_axe_tree = [0.05 0.05 0.9 0.9];

ax_tree = axes('Visible','on',...
               'XLim',[-0.05 1.05],...
               'YLim',[-0.05 1.05],...
               'Units','normalized',...
               'Position',pos_axe_tree,...
               'XTicklabelMode','manual',...
               'YTicklabelMode','manual',...
               'XTicklabel',[],'YTicklabel',[],...
               'XTick',[],'YTick',[],...
               'Box','On');

% draw lines of active nodes (nodes with entropy ~= NaN)

axes(ax_tree)

% first 2 nodes (root = top node)

%line('XData',[xpos(1) xtpos xpos(2)],'YData',[ypos(1) ytpos ypos(2)],'Color',line_color_tp);
line('XData',[xpos(1) xtpos],'YData',[ypos(1) ytpos],'Color',line_color_tp);
line('XData',[xtpos xpos(2)],'YData',[ytpos ypos(2)],'Color',line_color_hp);

% remaining nodes

for n = 3:2:Nnodes-1
   if ~isnan(D.entr(n))
      xt = xpos(D.root(n));
      yt = ypos(D.root(n));
%      line('XData',[xpos(n) xt xpos(n+1)],'YData',[ypos(n) yt ypos(n+1)],'Color',line_color_tp);
      line('XData',[xpos(n) xt],'YData',[ypos(n) yt],'Color',line_color_tp);
      line('XData',[xt xpos(n+1)],'YData',[yt ypos(n+1)],'Color',line_color_hp);
   end
end

% draw node labels

axes(ax_tree)

% top node comes first

text('String','(0,0)','FontSize',txt_font,'Position',[xtpos ytpos],'Color',txt_color,...
   'HorizontalAlignment','center','VerticalAlignment','middle','Clipping','on');

% remaining nodes

m = 1;
for n = 1:N
   d_str = int2str(n);
   for k = 0:2^n-1
      if ~isnan(D.entr(m))
         p_str = int2str(k);
         if n > 5
            txt_font = 6;
         elseif n > 4
            txt_font = 8;
         end
         text('String',['(' d_str ',' p_str ')'],...
            'FontSize',txt_font,...
            'Position',[xpos(m) ypos(m)],...
            'Color',txt_color,...
            'HorizontalAlignment','center',...
            'VerticalAlignment','middle',...
            'Clipping','on');
      end
      m = m + 1;
   end
end
legend('low','high');
zoom(ftree,'on');