function h = qmf_2(N,fs,alpha)
% function h = qmf_2(N,fs,alpha)
%
% design of 2-channel FIR QMF-bank, typ. h = qmf_2(50,0.65,0.1);
% (linear phase filter bank)
%
% N            FIR filter length (forced to be even)
% fstop        stop-band edge frequency normalized to Fsampl/2 (fstop > 0.5)
% alpha        weighting factor of stop band error energy (<= 1)
% h            column vector of filter impulse responses h = [h_lp;h_hp;g_lp;g_hp]
%              h_lp, h_hp   analysis filter impulse responses
%              g_lp, g_hp   synthesis filter impulse responses
%
% G. Doblinger, TU-Wien, 26.3.96, 22-12-97

% reference:
% C-K. Chen, J-H. Lee, "Design of Quadrature Mirror Filters with Linear Phase
% in the Frequency Domain", IEEE Trans. CAS-II, Vol. 39, Sept. 1992, pp. 593-605

if nargin == 0
   help qmf_2
   return
elseif fs <= 0.5 || fs >= 1
   error('fstop must be in (0.5,1)');
elseif alpha < 0
   error('alpha must not be less than 0');
end

if rem(N,2)
   N = N + 1;
end

% compute starting solution

fc = 0.5;                              % pass band edge (norm. to Fsampl/2)
df = fs-fc;

as = 7.95 + 7.18*N*df;                 % estimated stop band attenuation in dB
if as < 21                             % determine parameter beta
   beta = 0;
elseif as < 50
   beta = 0.58417*(as-20.96)^0.4 + 0.07886*(as-20.96);
else
   beta = 0.1102*(as-8.7);
end
h1 = fir1(N-1,fc,kaiser(N,beta));
h1 = sqrt(2)*h1(:);

Nh = N/2;
h0 = h1(1:Nh);                      % first half of symetric impulse response
L = 128;                            % number of frequency points in [0,fsampl/2]
theta = linspace(0,pi,L)';
n = (0:Nh-1) - Nh + 0.5;
Ut = 2*cos(theta*n);
Utpi = 2*cos((theta+pi)*n);

% compute matrix Q for stop band energy Es = h0'*Q*h0 of prototype filter

Lh1 = ceil((N-1)/2);
i = (1:Lh1)';
k = i(:,ones(1,Lh1));
ik = k - k';                        % matrix of indices i-k
fspi = fs*pi;
Q = -2*sin(ik*fspi)./(ik+eps);
ik = N+1-k-k';                      % matrix of indices N+1-i-k
Q = Q - 2*sin(ik*fspi)./ik;
Q = Q + 2*(pi-fspi)*eye(Lh1);       % modify diagonal elements
tau = 0.7;                          % weighting factor for impulse response update
tau1 = 1 - tau;
I1 = ones(1,Nh);
epsil = 1e-10;                      % error limit to terminate iteration
for k = 1:50
   H1 = Ut*h0;
   H2 = Utpi*h0;
   U = H1(:,I1).*Ut + H2(:,I1).*Utpi;           % matrix for pass band energy
   g0 = (U'*U + alpha*Q) \ sum(U)';             % solution for synthesis low pass filter
   h0 = tau1*h0 + tau*g0;                       % impulse response update
   err = norm(h0-g0)/norm(h0);                  % impulse response error
   if err < epsil, break, end
end
disp([int2str(k),' iterations, error = ', num2str(err)]);

h0 = sqrt(2)*h0;
h1 = [h0 ; h0(Nh:-1:1)];
h2 = h1;
h2(2:2:N) = -h2(2:2:N);                         % h2(n) = (-1)^n * h1(n)
g1 = h1;
g2 = -h2;
h = [h1;h2;g1;g2];

% compute reconstruction error

Nx = 1024;
Nx2 = Nx/2;
x = perinois(Nx,Nx2,[],sqrt(2));                   % create periodic white noise
x = x(:).';
Y = [filter(h1,1,x) ; filter(h2,1,x)];             % analysis filter bank
U = zeros(size(Y));
m = 1:2:Nx;
U(:,m) = Y(:,m);                                   % down/upsampling
y = filter(g1,1,U(1,:)) + filter(g2,1,U(2,:));     % synthesis filter bank
y = y(Nx2+1:Nx);                                   % take second period to avoid filter
                                                   % settling time

% compute and plot transfer function

close all
f = linspace(0,1,Nx2);

magH1 = abs(freqz(h1,1,pi*f));
magH1 = magH1/max(magH1);
eps1 = 1e-6;
i = find(magH1 <= eps1);
if ~isempty(i)
   magH1(i) = eps1 * ones(size(i));
end
magH2 = abs(freqz(h2,1,pi*f));
magH2 = magH2/max(magH2);
i = find(magH2 <= eps1);
if ~isempty(i)
   magH2(i) = eps1 * ones(size(i));
end
magY = fft(y);
magY = abs(magY(1:Nx2/2));
fy = linspace(0,1,Nx2/2);

i = find(magY <= eps1);
if ~isempty(i)
   magY(i) = eps1 * ones(size(i));
end

pos = [0.5,0.5,0.49,0.45];
figure('name','Time domain','Units','normal','Position',pos);
xstr = 'Time index n';
n = 0:N-1;
Nd_lp = (N-1)/2;
Nd_hp = N - Nd_lp - 1;
ystr1 = 'h_{lp}(n)';
ystr2 = 'h_{hp}(n)';
tstr1 = ['Analysis low pass filter impulse response, N_{lp} = ',int2str(N),...
         ', delay = ', num2str(Nd_lp), ' samples'];
tstr2 = ['Analysis high pass filter impulse response, N_{hp} = ',int2str(N),...
         ', delay = ', num2str(Nd_hp), ' samples'];
subplot(2,1,1), stem(n,h1), ylabel(ystr1), title(tstr1),
subplot(2,1,2), stem(n,h2), xlabel(xstr), ylabel(ystr2), title(tstr2),

pos = [0.005,0.5,0.49,0.45];
ftf = figure('name','Frequency domain (y-zoom on)','Units','normal','Position',pos);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
tstr1 = 'QMF-bank transfer functions (linear phase filters)';
tstr2 = ['Overall response (periodic white noise input), filter bank delay = ',...
         int2str(N-1), ' samples'];
subplot(2,1,1), plot(f,20*log10(magH1),'b',f,20*log10(magH2),'r'), ylabel(ystr),
   title(tstr1), grid on
legend('low pass','high pass');   
subplot(2,1,2), plot(fy,20*log10(magY),'b'), xlabel(xstr), ylabel(ystr),
   title(tstr2), grid on
zoom(ftf,'yon');