function D = wpspec(x,N,wname,treetype)
% function D = wpspec(x,N,wname,treetype)
%
% wavelet packet spectral analysis (scalogram analysis)
% NOTE: wavelet toolbox nedded for wavelet supplied by name, e.g. wname='coif4'
% use qmf_2.m in this case 
%
% x         signal vector
% N         number of analysis levels then best tree is computed
% wname     wavelet type in quotes ('coif4' typ.)
%           or filter vector h designed with qmf_2.m
% treetype  'b' for best basis tree, 'f' for full binary tree, 'w' for wavelet tree
%           ('f' if omitted)
% D         structure of node data (for best basis tree)
%           D.coef            node coefficients concatenated to row vector
%           D.start, D.stop   start, stop index of coeff. of each node
%           D.root            index of root node of each node (top node = index 0)
%           D.entr            entropy of each node (NaN, if node is omitted)
%           D.engy            energy of each node
%           D.delay           coefficient delay for each node
%
% examples: D = wpspec(sin(pi/200*[0:199].^2),4,'coif4','b');
%           h = qmf_2(50,0.65,0.1);
%           D = wpspec(sin(pi/400*[0:199].^2),4,h,'w');
%
% G. Doblinger. TU-Wien, 12-97, 11-2009

if nargin == 0
   help wpspec
   return
elseif nargin == 3
   treetype = 'f';
end

Ndata = length(x);

% determine filters, delays (centers of energy) of decomposition filters

if ischar(wname)
   if exist('wfilters','file') == 0
      disp('INFO: wfilters() not found, missing MATLAB wavelet toolbox');
      disp('      use qmf_2.m to create reference filter');
      D = [];
      return;
   end
   [h_lp,h_hp] = wfilters(wname,'d');
else
   Lh = length(wname)/4;
   h_lp = wname(1:Lh);
   h_hp = wname(Lh+1:2*Lh);
end

% compute wavelet packet decomposition

D = CompleteTree1(x,h_lp,h_hp,N);         % compute node coefficients of full binary
                                          % tree
if lower(treetype) == 'b'
   D = BestBasisTree(D);                  % find best basis tree structure
   treestr = 'Best wavelet packet tree adapted to signal';
elseif lower(treetype) == 'w'
   D = WaveletTree(D);
   treestr = 'Wavelet tree';
else
   treestr = 'Full binary tree';
end

% sort terminal nodes according to ascending frequencies

term_nodes = TermNodes(D);                % get terminal nodes
Ntn = length(term_nodes);
[depth,posi] = WavTreeLabel(term_nodes);
fnode = posi./(2.^depth);                 % assign a frequency to each node
[~,ifreq] = sort(fnode);
term_nodes = term_nodes(ifreq);
[depth,posi] = WavTreeLabel(term_nodes);

% create coefficient matrix for image plot of time-frequency tiling

Nf = 2^N;                                 % number of frequency bands
Cmat = zeros(Ndata,Nf);
m = 1;
if Ntn > 1                                % more than 1 termination node
   Nc = ceil(length(x)/2);
   Cmat1 = zeros(Nc,Ntn);                 % matrix to hold coefficient traces
   for k = 1:Ntn
      M = 2^depth(k);                     % coefficient decimation factor
      Lf = Nf/M;
      m1 = m + Lf;
      Nc = floor(Ndata/M);                % length of decimated coefficients
      coef = D.coef(D.start(term_nodes(k)):D.stop(term_nodes(k)));
      Cmat1(1:length(coef),k) = coef(:);
      C = coef(ones(M,1),:);              % hold operation (interpolate coefficients)
      coef = C(:);
      del = M*D.delay(term_nodes(k));
      coef = coef(del+1:M*Nc+del);        % remove coefficient delay
      Cmat(1:length(coef),m:m1-1) = coef(:,ones(1,Lf));
      m = m1;
   end
else
   coef = D.coef(D.start(1):D.stop(1));
   Cmat(1:length(coef),:) = coef(:,ones(1,Nf));
   Cmat1 = coef(:);
end

% plot result

close all
upperright = [0.55,0.3,0.44,0.65];
figure('name','Time-Frequency tiling','Units','normal','Position',upperright);

n = (0:Ndata-1)';
subplot(2,1,1), plot(n,x,'b'), xlabel('n'), ylabel('x[n]'), grid on
subplot(2,1,2), imagesc(abs(Cmat')), colormap(jet);
xlabel('Time index n'), ylabel('Frequency band index (scale)');
title(['Terminal coefficient magnitudes (scalogram), ',int2str(N), ' level tree']);
if N > 5
   ytvec = 0:4:Nf;
elseif N > 3
   ytvec = 0:2:Nf;
else
   ytvec = 0:Nf;
end
set(gca,'YDir','normal','YTick',ytvec);

% plot tree and terminal node coefficient traces

pos = [0.005,0.55,0.5,0.4];

if Ntn > 1
   PlotWavTree(D,pos,treestr);
end

NCr = size(Cmat1,1);
Cmat1 = Cmat1/max(max(abs(Cmat1)));
Cmat1 = Cmat1 + ones(NCr,1)*(1:Ntn);         % add offset for plotting

lowerleft = [0.005,0.04,0.5,0.4];
figure('name','Plot of terminal note coefficients','Units','normal',...
   'Position',lowerleft);

YTLmat = [];
RGBmat = [];
redstep = ones(1,Ntn);
greenstep = linspace(0,1,Ntn);
bluestep = linspace(1,0,Ntn);
for n = 1:Ntn
   YTLmat = [YTLmat ; sprintf('(%2d,%2d)',depth(n),posi(n))];
   RGBmat =  [RGBmat ; [redstep(n),greenstep(n),bluestep(n)]];
end

k = 1:NCr;
axh = axes('Box','on','YTick',(1:Ntn),'YTickLabel',YTLmat,'XLim',[0,NCr],'Ylim',[0,Ntn+1]);
for n = 1:Ntn
   line(k,Cmat1(:,n),'Color',RGBmat(n,:));
end
if Ntn > 10
   set(axh,'FontSize',8);
end
set(gca,'xlabel',text(0.5,-0.1,'Index k',...
   'Units','normal','HorizontalAlignment','center'));
set(gca,'ylabel',text(-0.15,0.4,'Node label',...
   'Units','normal','VerticalAlignment','middle','Rotation',90));
set(gca,'title',text(0.5,1.05,'Terminal coefficient traces',...
   'Units','normal','HorizontalAlignment','center'));


% -------------------------------------------------------------------------------------

function [d,p] = WavTreeLabel(nodeindex)
%function [d,p] = WavTreeLabel(nodeindex)
%
% convert binary wavelet tree node index to depth and position label
%
% nodeindex index (or index vector) of node (0 = top node)
% d         depth index (vector)
% p         position index (vector)
%

% Db, 29.12.97 (based on ind2depo() of wavelet toolbox)

d = zeros(size(nodeindex));
p = zeros(size(nodeindex));
k = find(nodeindex > 0);
d(k) = floor(log2(nodeindex(k)+1));
p(k) = nodeindex(k) - 2.^d(k) + 1;
