% spectrum analysis example
% show problems of applying windows, selecting FFT length, computing
% signal power, handle noise disturbances
%
% experiments:
% 1. FFT length Nfft = signal length, signal length is a multiple of period
% 2. show windowing effect by using a different Nfft
% 3. show high resolution spectral analysis of short data
% 4. show influence of high additive white gaussian noise
%
% G. Doblinger, TU-Wien, 11-2009

clear all
close all
clc

% create a sum of sinusoids 
%        Nsin
% x[n] = sum  A_k*sin(2pi*f_k/Fs*n + phi_k)
%        k=1
% use f = [400 1200 1300]' to show resolution of closely spaced sinusoids

Fs = 16000;
f = [400 1200 1300]';
A = [1 1 1]';
phi = [0 pi/4 pi/3]';

% determine periods of sinusoids, and sum of sinusoids
% Note: Fs and f must be integer-valued since gcd() expects integer values

Nsin = length(f);
Npsum = 1;
for n = 1:Nsin
   Np = Fs/gcd(Fs,f(n));         % period of each sinusoid
   Npsum = lcm(Npsum,Np);        % determine least common multiple
end

fprintf(1,'period length of sum of sinusoids = %d\n', Npsum);

M = 4;                           % number of periods used in Nx = length(x)

Nx = M*Npsum;                    % signal length is a multiple of period
n = 0:Nx-1;
X = A(:,ones(1,Nx)).*sin(2*pi/Fs*(f*n)+phi(:,ones(1,Nx)));
x = sum(X);
x = x(:);
pow_x = norm(x)^2/Nx;
fprintf(1,'signal power = %f, 0.5*sum(A^2) = %f\n', pow_x, 0.5*norm(A)^2);

% compute spectral magnitude using different time windows
% Note: if Nfft = Nx, then we get spectral lines in case of a rectangular
%       window because signal is periodic with Nfft

Nfft = Nx;                       % FFT length = signal length   
%Nfft = 2^(ceil(log2(Nx)));       % use power of 2 FFT length and
                                 % zero-padding 
                                 
Nfh = Nfft/2+1;
Xmag_r = abs(fft(x,Nfft));       % no window means rectangular window

fprintf(1,'spectral power = %f\n',norm(Xmag_r)^2/(Nfft*Nx));

win = 'chebwin';
%win = 'hamming';
eval(['w = ',win,'(Nx);']);      % e.g. win = chebwin(Nx);
                                 % we use eval() because we need the window
                                 % name in the legend of the plots
Xmag_ch = abs(fft(x.*w,Nfft));

% plot results

pos = [0.1 0.1 0.8 0.8];
figure('Units','normal','Position',pos);

fp = linspace(0,Fs/2,Nfh);       % vector of frequency points in [0,Fs/2]
subplot(3,1,1), plot(n,x), grid on;
xlabel('n'), ylabel('x[n]');
title(sprintf('sum of %d sinusiods, period = %d, number of periods = %d',...
   Nsin,Npsum,M));

Xmag_r = Xmag_r(1:Nfh);          % only Nhf freq. points needed due to
                                 % spectral symmetry of real-valued signals
Xmag_ch = Xmag_ch(1:Nfh);
subplot(3,1,2), plot(fp,Xmag_r,'r',fp,Xmag_ch,'b',...
   f,zeros(1,Nsin),'ko'), grid on;
xlabel('f in Hz'), ylabel('spectral mag.');
title(sprintf('FFT length = %d',Nfft));
legend('rectangular',win);

XdB_r = 20*log10(Xmag_r+eps);
dBmin = -100;
XdB_r = max(XdB_r,dBmin);        % limit spectral floor   
XdB_ch = 20*log10(Xmag_ch+eps);
XdB_ch = max(XdB_ch,dBmin);
subplot(3,1,3), plot(fp,XdB_r,'r',fp,XdB_ch,'b'), grid on;
xlabel('f in Hz'), ylabel('spectral mag. in dB');

%%
%
% repeat with short data length
% 
% demonstrate high resolution spectral analysis with MUSIC algorithm
% show that FFT spectral analysis fails to resolve closely spaced sinusoids

clc;
Nxs = ceil(Npsum/2);             % short data length (smaller than period)
if Nxs <= 2*Nsin+1
   fprintf(1,'data length <= 2*number of sinusoids + 1 = %d\n',2*Nsin+1);
   return
end

xs = x(1:Nxs);                   % select a short signal frame

close all;
pos = [0.1 0.1 0.8 0.8];
figure('Units','normal','Position',pos);

subplot(3,1,1), plot((0:Nxs-1),xs), grid on;
xlabel('n'), ylabel('x[n]');
title(sprintf('sum of %d sinusiods, period = %d, Nx = %d',...
   Nsin,Npsum,Nxs));

p = 2*Nsin;                      % number of spectral lines
                                 % Note: a real-valued sinusoid has
                                 % 2 spectral lines
[Xs,Rxs] = corrmtx(xs,p,'covariance');  % estimate correlation matrix
                                        % (covariance method)
[Sx,fp1] = pmusic(Xs,p,Nfft,Fs); % pseudospectrum using MUSIC

disp('eigenvalues of correlation matrix of noisefree data:')
disp(eig(Rxs));

f_est = rootmusic(Xs,p,Fs);      % estimate frequencies of sinusoids
f_est = f_est(f_est>0);
disp('estimated frequencies in Hz:');
disp(f_est);

Xmag_rs = abs(fft(xs,Nfft));
Xmag_rs = Xmag_rs(1:Nfh);
Xr = (Xmag_rs/max(Xmag_rs)).^2;
Sx = Sx/max(Sx);

subplot(3,1,2), plot(fp,Xr,'r',fp1,Sx,'b',...
   f,zeros(1,Nsin),'ko'), grid on;
xlabel('f in Hz'), ylabel('norm. spectral power');
title(sprintf('FFT length = %d',Nfft));
legend('FFT','pseudo spect.');

XrdB = 10*log10(Xr+eps);
SxdB = 10*log10(Sx+eps);
SxdB = max(SxdB,dBmin);
subplot(3,1,3), plot(fp,XrdB,'r',fp1,SxdB,'b'), grid on;
xlabel('f in Hz'), ylabel('spectral mag. in dB');
ht = text(0,dBmin-40,sprintf('f_e = %4.2f Hz  ',f_est));
set(ht,'FontSize',10,'FontName','Arial');
axis([0 Fs/2 dBmin 10]);

%%
%
% add white gaussian noise to original signal
%
% show noise performance of FFT and of MUSIC algorithm

clc;
SNR = -5;                        % SNR in dB
                                 % typical breakdown at -8 dB
v = randn(size(x));
gain = 10^(-SNR/20)*norm(x)/norm(v);
xn = x + gain*v;                 % at noise with specified SNR

close all;
pos = [0.1 0.1 0.8 0.8];
figure('Units','normal','Position',pos);

subplot(3,1,1), plot((0:Nx-1),xn), grid on;
xlabel('n'), ylabel('x[n]');
title(sprintf('sum of %d sinusiods plus noise, SNR = %3.2f dB, Nx = %d',...
   Nsin,SNR,Nx));

pcorr = floor(Nx/4);             % correlation matrix order
[Xn,Rxn] = corrmtx(xn,pcorr,'covariance');
p = 2*Nsin;                     
[Sxn,fp1] = pmusic(Xn,p,Nfft,Fs);  % pseudospectrum using MUSIC

disp('eigenvalues of correlation matrix of noisy data:')
disp(eig(Rxn));

f_est_n = rootmusic(Xn,p,Fs);    % estimate frequencies of sinusoids
f_est_n = f_est_n(f_est_n>0);
disp('estimated frequencies in Hz:');
disp(f_est_n);

Xmag_rn = abs(fft(xn,Nfft));
Xmag_rn = Xmag_rn(1:Nfh);
Xrn = (Xmag_rn/max(Xmag_rn)).^2;
Sxn = Sxn/max(Sxn);

subplot(3,1,2), plot(fp,Xrn,'r',fp1,Sxn,'b',...
   f,zeros(1,Nsin),'ko'), grid on;
xlabel('f in Hz'), ylabel('norm. spectral power');
title(sprintf('FFT length = %d',Nfft));
legend('FFT','pseudo spect.');

XrndB = 10*log10(Xrn+eps);
SxndB = 10*log10(Sxn+eps);
dBmin_n = -50-SNR;
SxndB = max(SxndB,dBmin_n);
subplot(3,1,3), plot(fp,XrndB,'r',fp1,SxndB,'b'), grid on;
xlabel('f in Hz'), ylabel('spectral mag. in dB');
title(sprintf('corr. matrix dimension = %d',pcorr+1));
ht = text(0,dBmin_n-20,sprintf('f_e = %4.2f Hz  ',f_est_n));
set(ht,'FontSize',10,'FontName','Arial');
axis([0 Fs/2 dBmin_n 10]); 




